@@ -7,9 +7,7 @@ pub mod meta;
 pub mod openid;
 pub mod registry;
 
-mod context;
 mod serde;
 mod translator;
 
-pub use context::*;
 pub use translator::*;
