@@ -3,6 +3,7 @@ use crate::{
     openid::{Credentials, TokenProvider},
 };
 use async_trait::async_trait;
+use std::fmt::{Debug, Formatter};
 
 /// A token provider, using an Access Token as static token.
 #[derive(Clone)]
@@ -11,6 +12,15 @@ pub struct AccessTokenProvider {
     pub token: String,
 }
 
+impl Debug for AccessTokenProvider {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("AccessTokenProvider")
+            .field("user", &self.user)
+            .field("token", &"***")
+            .finish()
+    }
+}
+
 #[async_trait]
 impl TokenProvider for AccessTokenProvider {
     type Error = reqwest::Error;
